
class DonutComponent extends React.Component {
	constructor(props) {
		super(props);
		if (this.isActive()) {
			window.onscroll = this.immediateClearPopup.bind(this);
			window.onclick = e => {
				if (!(e.target.className.indexOf('donut donut_') !== -1)) {
					this.immediateClearPopup.bind(this)();
				}
			};
		}
		this.showFirstTooltip = _.once(this.showFirstTooltip.bind(this));
	}
	showPopup(styles) {
		if (!this.isActive()) return;
		var _props = this.props;
		let info = _props.info,
		    store = _props.store,
		    isOnboarding = _props.isOnboarding;

		ReactDOM.render(React.createElement(
			ReactRedux.Provider,
			{ store: store },
			React.createElement(
				"div",
				{ style: styles, className: "donut donut_" + info.donutColor,
					onMouseLeave: this.clearPopup.bind(this), onMouseEnter: () => {
						clearTimeout(window.leaveTimer);
					} },
				isOnboarding && React.createElement(DonutTooltip, { childSafety: info.childSafety, color: this.props.info.donutColor, onboarding: info.onboarding, dispatch: this.props.dispatch }),
				React.createElement(
					"div",
					{ className: "donut__popover" },
					React.createElement(
						"div",
						{ className: "donut__header" },
						React.createElement(
							"span",
							null,
							truncateString(WotApi.getDecodedDomain(info.host), 30, 20)
						),
						React.createElement("div", { className: "donut__header__btn", onClick: this.onSettingsClick.bind(this) })
					),
					info.childSafety === NOT_SAFE && React.createElement(
						"div",
						{ className: "child-label" },
						webextApi.i18n.getMessage("NotAppropriate")
					),
					React.createElement(
						"div",
						{ className: "donut__body" },
						React.createElement(SecurityBubble, { safety: info.safety, color: info.bubbleColor }),
						React.createElement(ReputationBubble, { rating: +info.rating, confidence: +info.confidence })
					),
					React.createElement(
						"div",
						{ className: "donut__footer" },
						React.createElement("div", { className: "donut__logo", onClick: this.onLogoClick.bind(this) }),
						React.createElement(
							"a",
							{ className: "donut__viewaway", onClick: this.onLinkClick.bind(this) },
							webextApi.i18n.getMessage("viewFullScorecard") + " ►"
						)
					)
				)
			)
		), document.querySelector('.' + POPUP_CONTAINER));
	}
	showFirstTooltip(styles) {
		var _props2 = this.props;
		let info = _props2.info,
		    store = _props2.store,
		    isOnboarding = _props2.isOnboarding;

		ReactDOM.render(React.createElement(
			ReactRedux.Provider,
			{ store: store },
			React.createElement(
				"div",
				{ style: styles },
				React.createElement(DonutTooltip, { color: this.props.info.donutColor, onboarding: info.onboarding, dispatch: this.props.dispatch })
			)
		), document.querySelector('.' + POPUP_CONTAINER));
	}
	isOnboardingActiveDonut() {
		const onboardingTooltip = document.querySelector('.donut-tooltip.donut-tooltip_0');
		if (!onboardingTooltip) return false;
		const onboardRect = onboardingTooltip.getBoundingClientRect();
		const donutRect = this.donutEl.getBoundingClientRect();
		const diffX = Math.abs(onboardRect.x - donutRect.x);
		const diffY = Math.abs(onboardRect.y - donutRect.y);
		const isActive = diffX < 30 && diffY < 30;
		this.isActiveOnboardingDonut = isActive;
		return isActive;
	}
	isActive() {
		if (!this.props.isOnboarding) {
			return true;
		}
		if (this.props.info.onboarding.step >= 1 && this.props.info.onboarding.step < 3) {
			return this.isActiveOnboardingDonut;
		}
		return this.props.isOnboarding && this.isOnboardingActiveDonut();
	}
	onDonutOver(e) {
		if (!this.isActive()) return;
		let _e = _.merge({}, e),
		    position = "fixed",
		    left = this.getPosition(_e.currentTarget).left,
		    top = this.getPosition(_e.currentTarget).top;
		if (this.props.info.onboarding.step < 3) {
			position = "absolute";
			left = left - 4;
		}
		clearTimeout(window.leaveTimer);
		this.immediateClearPopup();
		this.props.info.onboarding.step === 0 && this.props.dispatch(toNextStepOnboarding());
		window.appearTimer = setTimeout(() => {
			this.showPopup({
				position: position,
				left: left + 'px',
				top: top + 'px',
				zIndex: '2147483646'
			});
			webextApi.runtime.sendMessage({
				name: SEND_MESSAGE, params: {
					[atob("ZWM=")]: "Search Popup",
					[atob("ZWE=")]: "Shown " + this.props.info.donutColor,
					[atob("ZWw=")]: this.props.info.host
				}
			});
		}, this.props.isOnboarding ? 200 : 400);
	}
	onDonutClick(e) {
		if (!this.isActive()) return;
		let _e = _.merge({}, e),
		    position = "fixed",
		    left = this.getPosition(_e.currentTarget).left,
		    top = this.getPosition(_e.currentTarget).top;
		if (this.props.info.onboarding.step < 3) {
			position = "absolute";
			left = left - 4;
		}
		clearTimeout(window.leaveTimer);
		this.immediateClearPopup();
		this.showPopup({
			position: position,
			left: left + 'px',
			top: top + 'px',
			zIndex: '2147483646'
		});
		webextApi.runtime.sendMessage({
			name: SEND_MESSAGE, params: {
				[atob("ZWM=")]: "Search Popup",
				[atob("ZWE=")]: "Shown " + this.props.info.donutColor,
				[atob("ZWw=")]: this.props.info.host
			}
		});
	}
	clearPopup() {
		if (document.querySelector('.' + POPUP_CONTAINER).innerHTML) {
			clearTimeout(window.appearTimer);
			window.leaveTimer = setTimeout(() => {
				this.immediateClearPopup();
			}, 500);
		} else {
			this.immediateClearPopup();
		}
	}
	immediateClearPopup(e) {
		if (!this.props.isOnboarding) {
			document.querySelector('.' + POPUP_CONTAINER).innerHTML = "";
			clearTimeout(window.appearTimer);
		}
	}
	onLinkClick() {
		if (!this.isActive()) return;
		webextApi.runtime.sendMessage({
			name: OPEN_TAB_MESSAGE,
			url: FULL_REPORT_URL + this.props.info.host + "?utm_source=addon&utm_content=search_donut"
		});
		webextApi.runtime.sendMessage({
			name: SEND_MESSAGE, params: {
				[atob("ZWM=")]: "Search Popup",
				[atob("ZWE=")]: "Full Scorecard " + this.props.info.donutColor,
				[atob("ZWw=")]: this.props.info.host
			}
		});
	}
	onLogoClick() {
		if (!this.isActive()) return;
		webextApi.runtime.sendMessage({
			name: SEND_MESSAGE, params: {
				[atob("ZWM=")]: "Search Popup",
				[atob("ZWE=")]: "WOT Logo " + this.props.info.donutColor,
				[atob("ZWw=")]: this.props.info.host
			}
		});
		webextApi.runtime.sendMessage({
			name: OPEN_TAB_MESSAGE,
			url: MAIN_PAGE_URL + "?utm_source=addon&utm_content=search_donut"
		});
	}
	onSettingsClick() {
		if (!this.isActive()) return;
		webextApi.runtime.sendMessage({
			name: SEND_MESSAGE, params: {
				[atob("ZWM=")]: "Search Popup",
				[atob("ZWE=")]: "Settings " + (this.props.info.donutColor.charAt(0).toUpperCase() + this.props.info.donutColor.substr(1).toLowerCase()),
				[atob("ZWw=")]: WotApi.getEncodedDomain(location.href)
			}
		});
		webextApi.runtime.sendMessage({
			name: OPEN_TAB_MESSAGE,
			url: webextApi.runtime.sendMessage({
				name: OPEN_TAB_MESSAGE,
				url: OPTIONS_URL + "?serpid=" + this.props.serpid
			})
		});
	}
	componentDidUpdate() {
		const allDonuts = [].slice.call(document.querySelectorAll('.donut'));
		const notGrayDonuts = allDonuts.filter(donEl => !~donEl.className.indexOf('donut_gray'));
		let donut = null;
		if (!notGrayDonuts[0] || !notGrayDonuts[0].getBoundingClientRect()) {
			return;
		} else if (this.getPosition(notGrayDonuts[0]).top <= 0) {
			donut = notGrayDonuts[1];
		} else {
			donut = notGrayDonuts[0];
		}
		if (!donut) return;
		this.props.info.onboarding.step === 0 && this.showFirstTooltip({
			position: "absolute",
			left: this.getPosition(donut).left + 23 + 'px',
			top: this.getPosition(donut).top - 14 + 'px',
			zIndex: '2147483646'
		});
	}
	getPosition(element) {
		if (typeof element === 'string') {
			element = document.querySelector(element);
		}
		const elemRect = element.getBoundingClientRect();
		let top = elemRect.top;
		let left = elemRect.left;
		// fix position for first step tutor
		if (this.props.isOnboarding) {
			const bodyRect = document.body.getBoundingClientRect();
			top -= bodyRect.top;
			left -= bodyRect.left;
		}
		return { top, left };
	}
	render() {
		var _props3 = this.props;
		let info = _props3.info,
		    containerSelector = _props3.containerSelector;

		if (info.donutColor === "invisible" || !info.settings.protection) {
			document.querySelector(containerSelector) && (document.querySelector(containerSelector).style.width = 0);
			document.querySelector(containerSelector) && (document.querySelector(containerSelector).style.height = 0);
			return React.createElement("i", null);
		}
		document.querySelector(containerSelector) && (document.querySelector(containerSelector).style.width = "16px");
		document.querySelector(containerSelector) && (document.querySelector(containerSelector).style.height = "16px");
		return React.createElement("div", {
			ref: donutEl => {
				this.donutEl = donutEl;
			},
			className: "donut donut_" + info.donutColor,
			onMouseOver: this.onDonutOver.bind(this),
			onMouseLeave: this.clearPopup.bind(this),
			onMouseUp: this.onDonutClick.bind(this)
		});
	}
}
DonutComponent.propTypes = {
	target: React.PropTypes.string.isRequired
};
function mapStateToPros(state, props) {
	props = this && this.props || props;
	let hostPage = WotApi.getEncodedDomain(window.location.href),
	    host = WotApi.getEncodedDomain(props.target),
	    donutsLevel = state.settings.donuts.all,
	    result = {
		info: {
			host: host,
			donutColor: 'invisible',
			bubbleColor: 'gray',
			safety: UNKNOWN,
			childSafety: 100,
			rating: -1,
			confidence: 0,
			onboarding: {},
			settings: state.settings
		}
	};
	for (let serpid in state.settings.donuts) {
		if (props.serpid == serpid) {
			donutsLevel = state.settings.donuts[serpid];
		}
	}
	if (typeof this === 'undefined' || donutsLevel === DONUTS_LEVEL_OFF) {
		return result;
	}
	if (state.ratings[host]) {
		let rating = state.ratings[host].rating,
		    confidence = state.ratings[host].confidence,
		    safety = state.ratings[host].safety,
		    color = getDonutColor(state.ratings[host], state.reviews[state.user && state.user.uid || 1][host]).color;
		result.info = {
			host: host,
			donutColor: donutsLevel === DONUTS_LEVEL_NORMAL && ['gray', 'green'].indexOf(color) >= 0 ? 'invisible' : color,
			bubbleColor: state.ratings[host].color,
			safety: safety,
			childSafety: state.ratings[host].childSafety,
			rating: rating,
			confidence: confidence,
			onboarding: state.onboarding,
			settings: state.settings
		};
	}
	return result;
}
mapStateToPros.dependsOnOwnProps = true;
DonutComponent = ReactRedux.connect(mapStateToPros)(DonutComponent);