const EMAIL_TEMPLATE = "__e-m-a-i-l__";
const LOGIN_TEMPLATE = "__n-a-m-e__";
const PASSWORD_TEMPLATE = "__p-a-s-s__";
const ADDRESS_TEMPLATE = "__a-d-d-r-e-s-s__";
const CARD_NUM_TEMPLATE = "__c-a-r-d__";
const PHONE_TEMPLATE = "__p-h-o-n-e__";
const LOCATION_TEMPLATE = "__l-o-c-a-t-i-o-n__";
const POSTCODE_TEMPLATE = "__p-o-s-t-c-o-d-e__";
const GENDER_TEMPLATE = "__g-e-n-d-e-r__";
const EMAIL_REGEX = /(([^<>()\[\]\.,;:\s@\"]+(\.[^<>()\[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})/i;
//we dont want to work with urls, which can contains those data
class PiFilter {
	constructor() {}
	parseUrlParams(paramsString) {
		let result = {};
		if (paramsString.substring(0, 1) == "?") {
			paramsString = paramsString.substring(1);
		}
		paramsString.split("&").forEach(function (part) {
			let item = part.split("=");
			result[item[0]] = decodeURIComponent(item[1]);
		});
		return result;
	}
	replaceParam(parsedParams, paramName, newValue) {
		if (paramName in parsedParams) {
			parsedParams[paramName] = newValue;
		}
		return parsedParams;
	}
	filterEmail(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "email", EMAIL_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "e-mail", EMAIL_TEMPLATE);
		for (let key in parsedParams) {
			if (EMAIL_REGEX.test(parsedParams[key])) {
				parsedParams[key] = EMAIL_TEMPLATE;
			}
		}
		return parsedParams;
	}
	filterName(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "nickname", LOGIN_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "firstname", LOGIN_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "lastname", LOGIN_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "surname", LOGIN_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "username", LOGIN_TEMPLATE);
		return parsedParams;
	}
	filterPassword(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "password", PASSWORD_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "pass", PASSWORD_TEMPLATE);
		return parsedParams;
	}
	filterCard(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "credit", CARD_NUM_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "creditcard", CARD_NUM_TEMPLATE);
		return parsedParams;
	}
	filterPhone(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "phone", PHONE_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "telephone", PHONE_TEMPLATE);
		return parsedParams;
	}
	filterLocation(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "location", LOCATION_TEMPLATE);
		return parsedParams;
	}
	filterGender(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "gender", GENDER_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "sex", GENDER_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "male", GENDER_TEMPLATE);
		parsedParams = this.replaceParam(parsedParams, "female", GENDER_TEMPLATE);
		return parsedParams;
	}
	filterAddress(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "address", ADDRESS_TEMPLATE);
		return parsedParams;
	}
	filterPostcode(parsedParams) {
		parsedParams = this.replaceParam(parsedParams, "postcode", POSTCODE_TEMPLATE);
		return parsedParams;
	}
	filterComponent(component) {
		let decodedComponent = decodeURIComponent(component);
		let parsedParams = this.parseUrlParams(decodedComponent);
		parsedParams = this.filterEmail(parsedParams);
		parsedParams = this.filterName(parsedParams);
		parsedParams = this.filterPassword(parsedParams);
		parsedParams = this.filterCard(parsedParams);
		parsedParams = this.filterPhone(parsedParams);
		parsedParams = this.filterLocation(parsedParams);
		parsedParams = this.filterGender(parsedParams);
		parsedParams = this.filterAddress(parsedParams);
		parsedParams = this.filterPostcode(parsedParams);
		return parsedParams;
	}
	verifyIsString(str) {
		return !!(str && (typeof str === "string" || str instanceof String));
	}
	filterAll(url) {
		if (!url || !this.verifyIsString(url)) {
			return null;
		}
		let parser = document.createElement('a');
		parser.href = url;
		//remove user and password from url
		if (parser.username != null && parser.username.length) {
			parser.username = LOGIN_TEMPLATE;
		}
		if (parser.password != null && parser.password.length) {
			parser.password = PASSWORD_TEMPLATE;
		}
		//remove pi from search params
		if (parser.search.length > 0) {
			try {
				let parsedParams = this.filterComponent(parser.search);
				//re-create the clean parameters search query
				let cleanSearch = "?";
				for (let key in parsedParams) {
					if (cleanSearch.length > 1) cleanSearch += "&";
					cleanSearch += key + "=" + parsedParams[key];
				}
				parser.search = cleanSearch;
			} catch (ex) {}
		}
		return parser.href;
	}
	filterAllArray(urlArray) {
		let filteredurlArray = [];
		if (!(urlArray instanceof Array)) {
			return null;
		}
		for (let url of urlArray) {
			let filteredUrl = this.filterAll(url);
			filteredurlArray.push(filteredUrl);
		}
		return filteredurlArray;
	}
	// filters the entire objects properties - doesn't create a copy of the destination object
	filterAllObject(obj) {
		let ignoredKeys = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

		for (var key in obj) {
			if (ignoredKeys.includes(key)) {
				continue;
			}
			if (obj[key] instanceof Array) {
				obj[key] = this.filterAllArray(obj[key]);
				continue;
			}
			obj[key] = this.filterAll(obj[key]);
		}
		return obj;
	}
}