
const browserName = crossbrowserName;
let webextApi = crossbrowser;
const isContent = !!crossbrowser.windows;
function truncateString(string) {
	let lengthInput = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 25;
	let lengthResult = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 15;

	if (string.length > lengthInput) {
		return string.slice(0, lengthResult) + '...' + string.slice(string.length - 6);
	}
	return string;
}
function upperFirstCase(string) {
	return string.charAt(0).toUpperCase() + string.slice(1);
}
function makeid() {
	let length = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;

	let text = "";
	let possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_";
	for (let i = 0; i < length; i++) text += possible.charAt(Math.floor(Math.random() * possible.length));
	return text;
}
function getAttr(href, name) {
	let url = new URL(href);
	return url.searchParams.get(name) || false;
}
function getDonutColor(rating, review) {
	let step = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : LAST_STEP;

	let categories = [],
	    userCategories = [],
	    color = rating && rating.color || 'gray',
	    warning = undefined;
	if (rating) {
		for (let cat of rating.categories) {
			cat && cat.id && categories.push(cat.id);
		}
		if (_.intersection(categories, CATEGORIES_RULES.icons.red).length > 0) {
			color = 'red';
			warning = 'severe';
		} else if (_.intersection(categories, CATEGORIES_RULES.icons.yellow).length > 0 || review && review.rating <= 60 && rating.confidence >= MIN_REPUTATION_CONFIDENCE && rating.rating <= 60) {
			color = 'yellow';
			warning = (rating.rating <= MAX_RED_STRIP_RATING ? 'red' : 'yellow') + 'strip';
		} else if (_.intersection(categories, CATEGORIES_RULES.icons.green).length > 0) {
			color = 'green';
			warning = '';
		}
	}
	if (review && review.step >= step) {
		for (let name of ['reputationCategories', 'safetyCategories', 'childSafetyCategories']) {
			for (let cat of review[name]) {
				cat && userCategories.push(cat);
			}
		}
		if (_.intersection(userCategories, CATEGORIES_RULES.icons.red).length > 0) {
			color = 'red-custom';
			warning = 'severe+caution';
		} else if (_.intersection(userCategories, CATEGORIES_RULES.icons.yellow).length > 0 || review && review.rating >= 0 && review.step >= step && review.rating <= 60) {
			color = 'yellow-custom';
			warning = 'caution';
		} else if (review && review.rating > 60 && review.step >= step) {
			color = 'green-custom';
			warning = '';
		}
	}
	if (_.intersection(categories, CATEGORIES_RULES.ratingFlow.childSafety).length > 0) {
		warning += '+parental';
	}
	return { color, warning };
}
function parseStatus(score) {
	let _status = 'Rookie';
	for (let status in ACTIVITY_SCORE_RULES) {
		if (score < ACTIVITY_SCORE_RULES[status]) {
			break;
		} else {
			_status = status;
		}
	}
	return _status;
}
function getFacebookSharingUrl() {
	let host = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
	let rating = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

	let text = webextApi.i18n.getMessage('shareTextFacebook' + (host ? 'Rate' : '')).replace("%LINK%", host ? FULL_REPORT_URL + host : FACEBOOK_SHARE_URL).replace("%WEBSITE%", WotApi.getDecodedDomain(host)).replace("%STARS%", (rating / 20).toFixed(0));
	return FACEBOOK_DIALOG_URL + "link=" + (host ? FULL_REPORT_URL + host + encodeURIComponent(FACEBOOK_RATING_PARAMETERS) : FACEBOOK_SHARE_URL) + "&description=" + text;
}
function getTwitterSharingUrl() {
	let host = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
	let rating = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

	let text = webextApi.i18n.getMessage('shareTextTwitter' + (host ? 'Rate' : '')).replace("%LINK%", host ? FULL_REPORT_URL + host + encodeURIComponent(TWITTER_RATING_PARAMETERS) : TWITTER_SHARE_URL).replace("%WEBSITE%", WotApi.getDecodedDomain(host)).replace("%STARS%", (rating / 20).toFixed(0));
	return TWITTER_DIALOG_URL + "&text=" + text + " via @web_of_trust";
}
function getGoogleSharingUrl() {
	let host = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
	let rating = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

	let text = webextApi.i18n.getMessage('shareTextGoogle' + (host ? 'Rate' : '')).replace("%LINK%", host ? FULL_REPORT_URL + host : GOOGLE_SHARE_URL).replace("%WEBSITE%", WotApi.getDecodedDomain(host)).replace("%STARS%", (rating / 20).toFixed(0));
	return GOOGLE_DIALOG_URL + "&url=" + (host ? FULL_REPORT_URL + host + encodeURIComponent(GOOGLE_RATING_PARAMETERS) : GOOGLE_SHARE_URL) + "&text=" + text;
}
function injectFonts() {
	let fonts = /*<non-firefox>*/document.createElement("link");
	fonts.href = FONTS_URL;
	fonts.rel = "stylesheet";
	document.querySelector('head').appendChild(fonts);
	/*fonts = /!*</non-firefox>*!/document.createElement("style");
 fonts.textContent = FONT_BASE64;
 document.querySelector('head').appendChild(fonts);*/
}
const punycode = {
	initial_n: 0x80,
	initial_bias: 72,
	delimiter: "\x2D",
	base: 36,
	damp: 700,
	tmin: 1,
	tmax: 26,
	skew: 38,
	maxint: 0x7FFFFFFF,
	/* decode_digit(cp) returns the numeric value of a basic code point
  (for use in representing integers) in the range 0 to base-1, or
  base if cp is does not represent a value. */
	decode_digit: function (cp) {
		return cp - 48 < 10 ? cp - 22 : cp - 65 < 26 ? cp - 65 : cp - 97 < 26 ? cp - 97 : base;
	},
	/* encode_digit(d,flag) returns the basic code point whose value
  (when used for representing integers) is d, which needs to be in
  the range 0 to base-1.  The lowercase form is used unless flag is
  nonzero, in which case the uppercase form is used.  The behavior
  is undefined if flag is nonzero and digit d has no uppercase
  form. */
	encode_digit: function (d, flag) {
		/* 0..25 map to ASCII a..z or A..Z
   26..35 map to ASCII 0..9 */
		return d + 22 + 75 * (d < 26) - ((flag != 0) << 5);
	},
	/* Bias adaptation function */
	adapt: function (delta, numpoints, firsttime) {
		var k;
		delta = firsttime ? Math.floor(delta / this.damp) : delta >> 1;
		delta += Math.floor(delta / numpoints);
		for (k = 0; delta > (this.base - this.tmin) * this.tmax >> 1; k += this.base) {
			delta = Math.floor(delta / (this.base - this.tmin));
		}
		return Math.floor(k + (this.base - this.tmin + 1) * delta / (delta + this.skew));
	},
	/* encode_basic(bcp,flag) forces a basic code point to lowercase if
  flag is zero, uppercase if flag is nonzero, and returns the
  resulting code point. The code point is unchanged if it  is
  caseless. The behavior is undefined if bcp is not a basic code
  point. */
	encode_basic: function (bcp, flag) {
		bcp -= (bcp - 97 < 26) << 5;
		return bcp + ((!flag && bcp - 65 < 26) << 5);
	},
	/* Main decode */
	decode: function (input, preserveCase) {
		/* Dont use uft16 */
		var output = [];
		var case_flags = [];
		var input_length = input.length;
		var n, out, i, bias, basic, j, ic, oldi, w, k, digit, t, len;
		/* Initialize the state: */
		n = this.initial_n;
		i = 0;
		bias = this.initial_bias;
		/* Handle the basic code points:  Let basic be the number of input
   code points before the last delimiter, or 0 if there is none,
   then copy the first basic code points to the output. */
		basic = input.lastIndexOf(this.delimiter);
		if (basic < 0) {
			basic = 0;
		}
		for (j = 0; j < basic; ++j) {
			if (preserveCase) {
				case_flags[output.length] = input.charCodeAt(j) - 65 < 26;
			}
			if (input.charCodeAt(j) >= 0x80) {
				throw new RangeError("decode: illegal input >= 0x80");
			}
			output.push(input.charCodeAt(j));
		}
		/* Main decoding loop:  Start just after the last delimiter if any
   basic code points were copied; start at the beginning
   otherwise. */
		for (ic = basic > 0 ? basic + 1 : 0; ic < input_length;) {
			/* ic is the index of the next character to be consumed, */
			/* Decode a generalized variable-length integer into delta,
    which gets added to i.  The overflow checking is easier
    if we increase i as we go, then subtract off its starting
    value at the end to obtain delta. */
			for (oldi = i, w = 1, k = this.base;; k += this.base) {
				if (ic >= input_length) {
					throw RangeError("decode: bad input (1)");
				}
				digit = this.decode_digit(input.charCodeAt(ic++));
				if (digit >= this.base) {
					throw RangeError("decode: bad input (2)");
				}
				if (digit > Math.floor((this.maxint - i) / w)) {
					throw RangeError("decode: overflow (1)");
				}
				i += digit * w;
				t = k <= bias ? this.tmin : k >= bias + this.tmax ? this.tmax : k - bias;
				if (digit < t) {
					break;
				}
				if (w > Math.floor(this.maxint / (this.base - t))) {
					throw RangeError("decode: overflow (2)");
				}
				w *= this.base - t;
			}
			out = output.length + 1;
			bias = this.adapt(i - oldi, out, oldi === 0);
			/* i was supposed to wrap around from out to 0, incrementing n
    each time, so we'll fix that now: */
			if (Math.floor(i / out) > this.maxint - n) {
				throw RangeError("decode: overflow (3)");
			}
			n += Math.floor(i / out);
			i %= out;
			/* Insert n at position i of the output: Case of last character
    determines uppercase flag: */
			if (preserveCase) {
				case_flags.splice(i, 0, input.charCodeAt(ic - 1) - 65 < 26);
			}
			output.splice(i, 0, n);
			i++;
		}
		if (preserveCase) {
			for (i = 0, len = output.length; i < len; i++) {
				if (case_flags[i]) {
					output[i] = String.fromCharCode(output[i]).toUpperCase().charCodeAt(0);
				}
			}
		}
		return toutf16.encode(output);
	},
	/* Main encode function */
	encode: function (input, preserveCase) {
		/* Bias adaptation function */
		var n, delta, h, b, bias, j, m, q, k, t, ijv, case_flags;
		if (preserveCase) {
			/* Preserve case, step1 of 2: Get a list of the unaltered
    string */
			case_flags = wot.idn.utf16.decode(input);
		}
		/* Converts the input in UTF-16 to Unicode */
		input = wot.idn.utf16.decode(input.toLowerCase());
		var input_length = input.length;
		/* Cache the length */
		if (preserveCase) {
			/* Preserve case, step2 of 2: Modify the list to true/false */
			for (j = 0; j < input_length; j++) {
				case_flags[j] = input[j] != case_flags[j];
			}
		}
		var output = [];
		/* Initialize the state: */
		n = this.initial_n;
		delta = 0;
		bias = this.initial_bias;
		/* Handle the basic code points: */
		for (j = 0; j < input_length; ++j) {
			if (input[j] < 0x80) {
				output.push(String.fromCharCode(case_flags ? this.encode_basic(input[j], case_flags[j]) : input[j]));
			}
		}
		h = b = output.length;
		/* h is the number of code points that have been handled, b is the
   number of basic code points */
		if (b > 0) {
			output.push(this.delimiter);
		}
		/* Main encoding loop: */
		while (h < input_length) {
			/* All non-basic code points < n have been handled already.
    Find the next larger one: */
			for (m = this.maxint, j = 0; j < input_length; ++j) {
				ijv = input[j];
				if (ijv >= n && ijv < m) {
					m = ijv;
				}
			}
			/* Increase delta enough to advance the decoder's <n,i> state
    to <m,0>, but guard against overflow: */
			if (m - n > Math.floor((this.maxint - delta) / (h + 1))) {
				throw RangeError("encode: overflow (1)");
			}
			delta += (m - n) * (h + 1);
			n = m;
			for (j = 0; j < input_length; ++j) {
				ijv = input[j];
				if (ijv < n && ++delta > this.maxint) {
					throw RangeError("encode: overflow (2)");
				}
				if (ijv == n) {
					/* Represent delta as a generalized variable-length
      integer: */
					for (q = delta, k = this.base;; k += this.base) {
						t = k <= bias ? this.tmin : k >= bias + this.tmax ? this.tmax : k - bias;
						if (q < t) {
							break;
						}
						output.push(String.fromCharCode(this.encode_digit(t + (q - t) % (this.base - t), 0)));
						q = Math.floor((q - t) / (this.base - t));
					}
					output.push(String.fromCharCode(this.encode_digit(q, preserveCase && case_flags[j] ? 1 : 0)));
					bias = this.adapt(delta, h + 1, h == b);
					delta = 0;
					++h;
				}
			}
			++delta;
			++n;
		}
		return output.join("");
	}
};
function toascii(target) {
	if (/[\[\:]/.test(target)) {
		return target;
		/* doesn't look like a proper name */
	}
	let result = [];
	target.toLowerCase().split(".").forEach(part => {
		if (/^[\w\-]*$/.test(part)) {
			result.push(part);
		} else {
			result.push("xn--" + punycode.encode(part));
		}
	});
	return result.join(".");
}
const toutf16 = {
	decode: function (input) {
		let output = [],
		    i = 0,
		    len = input.length,
		    value,
		    extra;
		while (i < len) {
			value = input.charCodeAt(i++);
			if ((value & 0xF800) === 0xD800) {
				extra = input.charCodeAt(i++);
				if ((value & 0xFC00) !== 0xD800 || (extra & 0xFC00) !== 0xDC00) {
					throw new RangeError("utf16.decode: illegal sequence");
				}
				value = ((value & 0x3FF) << 10) + (extra & 0x3FF) + 0x10000;
			}
			output.push(value);
		}
		return output;
	},
	encode: function (input) {
		let output = [],
		    i = 0,
		    len = input.length,
		    value;
		while (i < len) {
			value = input[i++];
			if ((value & 0xF800) === 0xD800) {
				throw new RangeError("utf16.encode: illegal value");
			}
			if (value > 0xFFFF) {
				value -= 0x10000;
				output.push(String.fromCharCode(value >>> 10 & 0x3FF | 0xD800));
				value = 0xDC00 | value & 0x3FF;
			}
			output.push(String.fromCharCode(value));
		}
		return output.join("");
	}
};
function tounicode(target) {
	if (!target) {
		return target;
	}
	let result = [];
	target.toLowerCase().split(".").forEach(function (part) {
		if (part.indexOf("xn--") == 0) {
			result.push(punycode.decode(part.replace("xn--", "")));
		} else {
			result.push(part);
		}
	});
	return result.join(".");
}
function getLocale() {
	let locale = chrome.i18n.getUILanguage().replace('_', '-');
	return locale.length < 7 ? locale : 'en-US';
}
function validateDomain(host) {
	return VALID_HOSTNAME.test(host) ? host : "";
}
function compareVersions(a, b) {
	// compare versions
	// Return 1 if a > b
	// Return -1 if a < b
	// Return 0 if a == b
	if (a === b) return 0;
	const aComponents = a.split('.');
	const bComponents = b.split('.');
	const len = Math.min(aComponents.length, bComponents.length);
	// loop while the components are equal
	for (let i = 0; i < len; i++) {
		// A bigger than B
		if (parseInt(aComponents[i]) > parseInt(bComponents[i])) return 1;
		// B bigger than A
		if (parseInt(aComponents[i]) < parseInt(bComponents[i])) return -1;
	}
	// If one's a prefix of the other, the longer one is greater.
	if (aComponents.length > bComponents.length) {
		return 1;
	} else if (aComponents.length < bComponents.length) {
		return -1;
	} else {
		// Otherwise they are the same.
		return 0;
	}
}
function getOS() {
	let os = null,
	    clientStrings = [{ s: 'Windows 10', r: /(Windows 10.0|Windows NT 10.0)/ }, { s: 'Windows 8.1', r: /(Windows 8.1|Windows NT 6.3)/ }, { s: 'Windows 8', r: /(Windows 8|Windows NT 6.2)/ }, { s: 'Windows 7', r: /(Windows 7|Windows NT 6.1)/ }, { s: 'Windows Vista', r: /Windows NT 6.0/ }, { s: 'Windows Server 2003', r: /Windows NT 5.2/ }, { s: 'Windows XP', r: /(Windows NT 5.1|Windows XP)/ }, { s: 'Windows 2000', r: /(Windows NT 5.0|Windows 2000)/ }, { s: 'Windows ME', r: /(Win 9x 4.90|Windows ME)/ }, { s: 'Windows 98', r: /(Windows 98|Win98)/ }, { s: 'Windows 95', r: /(Windows 95|Win95|Windows_95)/ }, { s: 'Windows NT 4.0', r: /(Windows NT 4.0|WinNT4.0|WinNT|Windows NT)/ }, { s: 'Windows CE', r: /Windows CE/ }, { s: 'Windows 3.11', r: /Win16/ }, { s: 'Android', r: /Android/ }, { s: 'Open BSD', r: /OpenBSD/ }, { s: 'Sun OS', r: /SunOS/ }, { s: 'Linux', r: /(Linux|X11)/ }, { s: 'iOS', r: /(iPhone|iPad|iPod)/ }, { s: 'Mac OS X', r: /Mac OS X/ }, { s: 'Mac OS', r: /(MacPPC|MacIntel|Mac_PowerPC|Macintosh)/ }, { s: 'QNX', r: /QNX/ }, { s: 'UNIX', r: /UNIX/ }, { s: 'BeOS', r: /BeOS/ }, { s: 'OS/2', r: /OS\/2/ }, { s: 'Search Bot', r: /(nuhk|Googlebot|Yammybot|Openbot|Slurp|MSNBot|Ask Jeeves\/Teoma|ia_archiver)/ }];
	for (let id in clientStrings) {
		let cs = clientStrings[id];
		if (cs.r.test(navigator.userAgent)) {
			os = cs.s;
			break;
		}
	}
	return os;
}
function getVersion() {
	let version = browserName === 'firefox' ? VERSION : webextApi.runtime.getManifest().version;
	;
	return version;
}
function getBrowserInfo() {
	let ua = navigator.userAgent,
	    tem,
	    M = ua.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i) || [];
	if (/trident/i.test(M[1])) {
		tem = /\brv[ :]+(\d+)/g.exec(ua) || [];
		return { name: 'IE', version: tem[1] || '' };
	}
	if (M[1] === 'Chrome') {
		tem = ua.match(/\b(OPR|Edge)\/(\d+)/);
		if (tem != null) return { name: tem[1].replace('OPR', 'Opera'), version: tem[2] };
	}
	M = M[2] ? [M[1], M[2]] : [navigator.appName, navigator.appVersion, '-?'];
	if ((tem = ua.match(/version\/(\d+)/i)) != null) M.splice(1, 1, tem[1]);
	return { name: M[0], version: M[1] };
}
function generateUuidV4() {
	let ts = Date.now();
	if (window.performance && window.performance.now) {
		ts += performance.now();
	}
	let str = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx';
	let uuid = str.replace(/[xy]/g, function (c) {
		let r = (ts + Math.random() * 16) % 16 | 0;
		ts = Math.floor(ts / 16);
		return (c === 'x' ? r : r & 0x3 | 0x8).toString(16);
	});
	return uuid;
}
function buildUrl(method, params) {
	let string = Object.keys(params).map(key => {
		return key + '=' + encodeURIComponent(params[key]);
	}).join('&');
	if (method === '') {
		return string;
	} else if (method.indexOf('?') === -1) {
		return method + '?' + string;
	} else {
		return method + string;
	}
}
function getDomain(href) {
	let l = document.createElement("a"),
	    regexp = /([^.]+\.)?([^\.]+\..+)/;
	l.href = href && href.indexOf("://") !== -1 ? href : "http://" + href;
	let matches = l.hostname.match(regexp);
	return matches && matches.length > 2 ? matches[2] : l.hostname;
}
function getRandomBool() {
	let a = new Uint8Array(1);
	crypto.getRandomValues(a);
	return a[0] > 127;
}
function checkLocalhost(url) {
	if (url.indexOf('//localhost/') !== -1 || url.indexOf('//localhost:') !== -1) {
		return true;
	}
	return false;
}
function firefoxRepaint() {
	if (browserName !== 'firefox') return;
	document.body.style.display = 'none';
	document.body.scrollHeight;
	document.body.style.display = 'block';
}