
let xmlToJson = xml => {
	let obj = {};
	if (xml.nodeType == 1) {
		if (xml.attributes.length > 0) {
			obj["@attributes"] = {};
			for (let j = 0; j < xml.attributes.length; j++) {
				let attribute = xml.attributes.item(j);
				obj["@attributes"][attribute.nodeName] = attribute.value;
			}
		}
	} else if (xml.nodeType == 3) {
		// text
		obj = xml.value;
	}
	if (xml.hasChildNodes()) {
		for (let i = 0; i < xml.childNodes.length; i++) {
			let item = xml.childNodes.item(i);
			let nodeName = item.nodeName;
			if (typeof obj[nodeName] == "undefined") {
				obj[nodeName] = xmlToJson(item);
			} else {
				if (typeof obj[nodeName].push == "undefined") {
					let old = obj[nodeName];
					obj[nodeName] = [];
					obj[nodeName].push(old);
				}
				obj[nodeName].push(xmlToJson(item));
			}
		}
	}
	return obj;
};
class _WotApi {
	constructor() {
		this.root = "https://api.mywot.com/0.5/";
		this.queryDevRoot = "http://localhost:8889/0.5/";
		this.methods = {
			register: {
				url: this.root + 'register?'
			},
			query: {
				url: (APIS_DEV_MODE ? this.queryDevRoot : this.root) + 'query'
			},
			links: {
				url: 'http://api.mywot.com'
			},
			getSerps: {
				url: "http://static-cdn.mywot.com/settings/extensions/serps.json"
			},
			comments: {
				url: 'https://scorecard.api.mywot.com/comments'
			},
			commentsCounter: {
				url: 'https://scorecard.api.mywot.com/comments/count'
			},
			rating: {
				url: 'https://scorecard.api.mywot.com/rating'
			},
			ratingTestimony: {
				url: 'https://scorecard.api.mywot.com/rating-testimony'
			}
		};

		this.crypto = new WotCrypto();
		this.rtSublast = "";
		this.witness = null;
		this.rtWitness = null;
		this.counters = {};
	}
	onError(xhr) {
		let params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

		//
		setTimeout(() => {
			window.location = location;
		}, 1000);
	}
	getRtSublast() {
		return this.rtSublast ? this.rtSublast : "";
	}
	setRtSublast(url) {
		if (this.isValidUrl(url)) {
			this.rtSublast = url;
		}
	}
	isValidUrl(url) {
		return typeof url === "string" && /^https?:\/\/(?!localhost)/.test(url) && url.indexOf("chrome/newtab") === -1;
	}
	isChromeInstantUrl(url) {
		return typeof url === "string" && url.indexOf("sourceid=chrome-instant") !== -1;
	}
	get(url) {
		let callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : xhr => {};
		let headers = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

		let xhr = new XMLHttpRequest(),
		    start = +new Date();
		xhr.open('GET', url);
		headers.forEach(h => xhr.setRequestHeader(h.name, h.value));
		xhr.onload = () => {
			xhr.time = +new Date() - start;
			return callback.call(null, xhr);
		};
		xhr.send();
	}
	post(url) {
		let params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
		let headers = arguments[2];
		let callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : xhr => {};

		let xhr = new XMLHttpRequest(),
		    start = +new Date();
		xhr.open('POST', url, true);
		if (typeof headers === 'function') {
			callback = headers;
			xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
		} else {
			xhr.setRequestHeader("Authorization", headers.authorization);
			xhr.setRequestHeader("Content-type", headers.contentType);
		}
		xhr.onload = () => {
			xhr.time = +new Date() - start;
			return callback.call(null, xhr);
		};
		xhr.send(params);
	}
	delete(url) {
		let params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
		let headers = arguments[2];
		let callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : xhr => {};

		let xhr = new XMLHttpRequest();
		xhr.open('DELETE', url);
		if (typeof headers === 'function') {
			callback = headers;
			xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
		} else {
			xhr.setRequestHeader("Authorization", headers.authorization);
			xhr.setRequestHeader("Content-type", headers.contentType);
		}
		xhr.onload = function () {
			return callback.call(null, xhr);
		};
		xhr.send(params);
	}
	//request all comments for target
	getCommentsData(params) {
		return new Promise((resolve, reject) => {
			const url = params.url;
			this.get(buildUrl(this.methods.comments.url, params), xhr => {
				if (xhr.status === 200 || xhr.status === 304) {
					const data = JSON.parse(xhr.responseText);
					const dataResult = [];
					if (!data.length) {
						resolve({ [url]: { data: dataResult } });
					} else {
						data.forEach(commentItem => {
							let rating = -1,
							    bubbles = 'none',
							    childSafety = true;
							commentItem.rating && commentItem.rating.forEach(ratingItem => {
								if (ratingItem.name === 0) {
									rating = ratingItem.t;
								} else if (ratingItem.name === 501 && ratingItem.vote === 1) {
									bubbles = 'green';
								} else if (ratingItem.vote === 1 && [202, 206].indexOf(ratingItem.name) !== -1) {
									bubbles = 'yellow';
								} else if (ratingItem.vote === 1 && [101, 103].indexOf(ratingItem.name) !== -1) {
									bubbles = 'red';
								} else if (ratingItem.vote === 1 && [401, 402, 403].indexOf(ratingItem.name) !== -1) {
									childSafety = false;
								}
							});
							dataResult.push({
								comment: commentItem.comment,
								name: commentItem.name,
								picture: commentItem.picture,
								status: commentItem.status,
								timestamp: commentItem.timestamp,
								uid: commentItem.uid,
								url: commentItem.url,
								wcid: commentItem.wcid,
								rating: rating,
								bubbles: bubbles,
								childSafety: childSafety
							});
						});
						resolve({ [url]: { data: dataResult } });
					}
				} else {
					// this.onError(xhr);
					reject(xhr.status);
				}
			});
		});
	}
	getOldWitnessSettings(settingKey, settingId) {
		let id = JSON.parse(localStorage.getItem(settingId)),
		    key = JSON.parse(localStorage.getItem(settingKey));
		return id && key ? { id, key } : false;
	}
	//request user review for target
	getUserRating(url, uid, token) {
		const params = {};
		params.url = url;
		params.uid = uid;
		params.salt = +new Date();
		const headers = [{ name: 'Authorization', value: "Bearer " + token }];
		return new Promise((resolve, reject) => {
			this.get(buildUrl(this.methods.ratingTestimony.url, params), xhr => {
				if (xhr.status === 200) {
					const data = JSON.parse(xhr.responseText);
					resolve([{ rating: _.isArray(data) ? data : [data] }]); // for compatibility in getUserCommentAndRating
				} else {
					;
					reject(new Error("Cannot request user review with" + xhr.status));
				}
			});
		}, headers);
	}
	//request comments and counter fro them
	comments(url) {
		let limit = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 20;
		let offset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

		let params = typeof url === 'string' ? { url, limit, offset } : url;
		params.includeRating = 'true';
		params.sort = '-upvotes';
		params.salt = +new Date();
		let comments = this.getCommentsData(params);
		let counter = new Promise((resolve, reject) => {
			this.get(buildUrl(this.methods.commentsCounter.url, { url: url, salt: +new Date() }), xhr => {
				if ((xhr.status === 200 || xhr.status === 304) && xhr.responseText !== '{}') {
					resolve({ [url]: { count: JSON.parse(xhr.responseText) } });
				} else {
					// this.onError(xhr);
					reject(xhr.status);
				}
			});
		});
		return Promise.all([comments, counter]);
	}
	//request user comments and rating
	getUserCommentAndRating(url, uid, token) {
		const params = {};
		params.includeRating = 'true';
		params.includeVotes = 'true';
		params.url = url;
		params.uid = uid;
		return new Promise((resolve, reject) => {
			this.get(buildUrl(this.methods.comments.url, params), xhr => {
				if (xhr.status === 200 || xhr.status === 304) {
					let apiData = JSON.parse(xhr.responseText);
					if (_.isEmpty(apiData)) {
						this.getUserRating(url, uid, token).then(resolve);
					} else {
						resolve(apiData);
					}
				} else {
					// this.onError(xhr);
					reject(xhr.status);
				}
			});
		});
	}
	//method to send user comment
	sendComments(content, token) {
		const headers = {
			'authorization': 'Bearer ' + token,
			'contentType': 'application/json'
		};
		this.post(this.methods.rating.url, JSON.stringify(content), headers, xhr => {
			if (xhr.status !== 200) {
				// this.onError(xhr);
			}
		});
	}
	//method to delete user comment
	deleteComments(url, token, uid) {
		let headers = {
			'authorization': 'Bearer ' + token,
			'contentType': 'application/json'
		};
		this.post(this.methods.rating.url, JSON.stringify({ target: url, comments: '', rating: -1, tags: [], uid }), headers, xhr => {
			if (xhr.status !== 200) {
				// this.onError(xhr);
			}
		});
		this.delete(this.methods.rating.url, JSON.stringify({ url }), headers, xhr => {
			if (xhr.status !== 200) {
				// this.onError(xhr);
			}
		});
	}
	//method to obtain basic auth key and id
	register(witness) {
		let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { isRt: false };

		if (witness && witness.id && witness.key) {
			if (options.isRt) {
				this.rtWitness = witness;
			} else {
				this.witness = witness;
			}
			return new Promise((resolve, reject) => {
				resolve(witness);
			});
		}
		// check for old settings
		if (options.isRt) {
			let oldRtWitnessSettings = this.getOldWitnessSettings("rt_witness_key", "rt_witness_id");
			if (oldRtWitnessSettings) {
				this.rtWitness = oldRtWitnessSettings;
				return new Promise((resolve, reject) => {
					resolve(oldRtWitnessSettings);
				});
			}
		} else {
			let oldWitnessSettings = this.getOldWitnessSettings("witness_key", "witness_id");
			if (oldWitnessSettings) {
				this.witness = oldWitnessSettings;
				return new Promise((resolve, reject) => {
					resolve(oldWitnessSettings);
				});
			}
		}
		return new Promise((resolve, reject) => {
			let apiname = "register",
			    witness = { id: "", key: "" },
			    nonce = this.crypto.getnonce(apiname, witness),
			    params = {
				nonce: nonce,
				lang: getLocale(),
				version: getVersion()
			};
			if (options.isRt) {
				params.wg = 1;
			}
			this.get(buildUrl(this.methods.register.url, params), xhr => {
				if (xhr.status === 200) {
					let response = xhr.responseText.match(/[0-9a-z]{40}/g),
					    id = response[0],
					    key = response[1],
					    witness = { id: id, key: key };
					if (options.isRt) {
						this.rtWitness = witness;
					} else {
						this.witness = witness;
					}
					resolve(witness);
				} else {
					;
					this.onError(xhr);
					reject(xhr.status);
				}
			});
		});
	}
	registerAll(witness, rtWitness) {
		let register = this.register(witness);
		let rtRegister = this.register(rtWitness, { isRt: true });
		return Promise.all([register, rtRegister]);
	}
	//method to request rating for each result on serp page
	links(urls, previous) {
		return new Promise((resolve, reject) => {
			let hosts = '';
			for (let url of urls) {
				if (!url) {
					continue;
				}
				let host = this.getEncodedDomain(url);
				if (hosts.indexOf(host) === -1 && host.indexOf('\.') !== -1) {
					hosts += host + '/';
				}
			}
			if (hosts.length < 0) {
				reject("Invalid urls");
			} else if (hosts.split("/").length == 2) {
				// console.log(hosts);
				// this.query(urls[0]).then((_result)=>{
				// 	resolve(previous ? _.merge(previous, _result) : _result);
				// })
				hosts += hosts;
			}
			if (hosts.length >= MAX_PARAMS_LENGTH) {
				this.links(urls.splice(0, urls.length / 2)).then(data => {
					this.links(urls, data).then(data => {
						resolve(data);
					});
				});
			} else {
				let apiname = "link",
				    nonce = this.crypto.getnonce(apiname, this.witness),
				    params = {
					hosts: this.crypto.encrypt(hosts, nonce, this.witness.key),
					id: this.witness.id,
					nonce: nonce,
					lang: getLocale(),
					version: getVersion()
				},
				    path = "/0.4/" + apiname + "?" + buildUrl('', params),
				    //TODO different url
				auth = this.crypto.authenticate(path, this.witness);
				path += "&auth=" + auth;
				this.get(this.methods.links.url + path, xhr => {
					if (xhr.status === 200) {
						let parser = new DOMParser(),
						    response = xmlToJson(parser.parseFromString(xhr.responseText, "application/xml")),
						    targets = response.link.target,
						    nonce = response.link['@attributes'].nonce,
						    result = {};
						if (targets && targets.length > 0) {
							for (let target of targets) {
								let host = this.crypto.decrypt(target['@attributes'].normalized, nonce, {
									index: target['@attributes'].index,
									response_str: true,
									witness_key: this.witness.key
								});
								result[host] = this.parseResponseRating(target, host);
							}
						}
						resolve(previous ? _.merge(previous, result) : result);
					} else {
						this.onError(xhr);
						reject(xhr.status);
					}
				});
			}
		});
	}
	//method to request rating for visited site
	query(targetParams) {
		return new Promise((resolve, reject) => {
			if (!targetParams || !targetParams.host || !this.rtWitness) {
				if (this.counters[targetParams.host]) {
					this.counters[targetParams.host]++;
				} else {
					this.counters[targetParams.host] = 1;
				}
				if (this.counters[targetParams.host] <= 10) {
					setTimeout(() => {
						resolve(this.query.bind(this)(targetParams));
					}, 100);
				} else {
					;
				}
				return reject(new Error('Cannot query' + JSON.stringify(targetParams)));
			}
			let host = targetParams.host;
			let apiname = "query";
			let nonce = this.crypto.getnonce(apiname, this.rtWitness);
			let params = {
				id: this.rtWitness.id,
				nonce: nonce,
				lang: getLocale(),
				version: getVersion() + "-" + RT_API_SUBVERSION
			};
			targetParams = Object.assign(targetParams, this.getAdditionalTargetParams());
			let targetParamsQuerystring = this.targetParamsToQuerystring(targetParams, {});
			params.target = this.crypto.encrypt(targetParamsQuerystring, nonce, this.rtWitness.key);
			let queryString = this.paramsToQuerystring(params, {});
			let topNonce = this.crypto.getnonce(apiname);
			let topParams = {
				target: this.crypto.encrypt(host, topNonce, this.witness.key),
				id: this.witness.id,
				nonce: topNonce,
				lang: getLocale(),
				version: getVersion()
			};
			let topQueryString = this.paramsToQuerystring(topParams);
			let path = "/0.4/" + apiname + "?" + topQueryString;
			topQueryString += "&auth=" + this.crypto.authenticate(path, this.witness);
			topQueryString += "&wg=1";
			topQueryString += "&b64=" + btoa(queryString);
			this.post(this.methods.query.url, topQueryString, xhr => {
				if (xhr.status === 200) {
					if (APIS_DEV_MODE) {
						reject(API_ERROR_CODES.E101); // dev mode is on - ratings are unavailable
						return;
					}
					let parser = new DOMParser(),
					    response = xmlToJson(parser.parseFromString(xhr.responseText, "application/xml")),
					    rating = this.parseResponseRating(response.query.target, host),
					    review = this.parseResponseReview(response.query.target);
					resolve({ rating: { [host]: rating }, review: { [host]: review } }); //TODO there are a lot of messages in response, maybe we need those
				} else {
					this.onError(xhr);
					reject(xhr.status);
				}
			});
		});
	}
	//method to send user review
	submit(target) {
		let review = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

		return new Promise((resolve, reject) => {
			let votes = "",
			    rating,
			    childSafety,
			    safety;
			if (review) {
				let categories = _.concat([], review.safetyCategories, review.childSafetyCategories, review.reputationCategories);
				rating = review.rating;
				if (review.safety === -1) {
					safety = -1;
				} else if (review.safety === true) {
					safety = 100;
				} else {
					safety = 39;
					for (let category of review.safetyCategories) {
						if (ALL_CATEGORIES[category].color == "red") {
							safety = 0;
						}
					}
				}
				if (review.childSafety === -1) {
					childSafety = -1;
				} else if (review.childSafety === true) {
					childSafety = 100;
				} else {
					childSafety = 39;
					for (let category of review.childSafetyCategories) {
						if (ALL_CATEGORIES[category].color == "red") {
							childSafety = 0;
						}
					}
				}
				for (let category in ALL_CATEGORIES) {
					votes += category + ':' + (_.indexOf(categories, +category) !== -1 ? 1 : 0) + "/";
				}
			} else {
				for (let category in ALL_CATEGORIES) {
					votes += category + ":-1/";
				}
				rating = -1;
				childSafety = -1;
			}
			let apiname = "submit",
			    nonce = this.crypto.getnonce(apiname, this.witness),
			    params = {
				target: this.crypto.encrypt(target, nonce, this.witness.key),
				id: this.witness.id,
				testimony_0: rating,
				testimony_4: childSafety,
				votes: votes,
				nonce: nonce,
				lang: getLocale(),
				version: getVersion()
			},
			    path = "/0.4/" + apiname + "?" + buildUrl('', params),
			    //TODO different url
			auth = this.crypto.authenticate(path, this.witness);
			path += "&auth=" + auth;
			this.get(this.methods.links.url + path, xhr => {
				if (xhr.status === 200) {
					resolve();
				} else {
					this.onError(xhr);
					reject(xhr.status);
				}
			});
		});
	}
	//method to obtain serps configuration
	getSerps() {
		return new Promise((resolve, reject) => {
			// console.log(this);
			this.get(buildUrl(this.methods.getSerps.url, { id: this.witness.id }), xhr => {
				if (xhr.status === 200) {
					let configs = JSON.parse(xhr.responseText);
					for (let config of configs) {
						if (config.id === 5) {
							config.ignoreParent.push("._5ya");
						}
					}
					resolve(configs);
				} else {
					// this.onError(xhr);
					reject(xhr.status);
				}
			});
		});
	}
	getEncodedDomain(href) {
		if (!href) return href;
		href = href && href.indexOf("://") !== -1 ? href : "http://" + href;
		href = href.replace(/^\s*/, "").replace(/\s*$/, "").replace('://www.', '://');
		let parser = document.createElement('a'),
		    host,
		    path;
		parser.href = href;
		host = parser.hostname;
		if (SUPPORTED_PROTOCOLS.test(href)) {
			let match = /^(\w+):\/\/(([^:@]+)(:([^@]+))?@)?([^:\/\?&=#\[\]]+|\[[^\/\?&=#\[\]]+\])\.?(:(\d+))?(.*)$/.exec(href);
			if (match && host) {
				host = toascii(host);
				return this.encodehostname(host, match[9]);
			} else {
				return host;
			}
		}
	}
	//get decoded domain
	getDecodedDomain(host) {
		try {
			let m = /^_p_([a-z2-7]+)\.(.+)$/.exec(host);
			if (!m || !m[1] || !m[2]) {
				return tounicode(host);
			}
			let decoded = this.crypto.base32.decode(m[1]);
			if (decoded == null) {
				return tounicode(host);
			}
			return tounicode(m[2]) + "/" + decoded;
		} catch (e) {
			;
		}
		return tounicode(host);
	}
	//calculate the color of bubbles
	parseCategories(categories) {
		let attr = {
			safety: UNKNOWN,
			color: "gray"
		};
		for (let category of categories) {
			if (_.indexOf(CATEGORIES_RULES.bubbles.notSafe, +category.id) !== -1 && category.confidence >= MIN_CATEGORY_CONFIDENCE) {
				attr.safety = NOT_SAFE;
				attr.color = 'red';
				return attr; //if site is "Not safe" - skip all other conditions
			} else if (_.indexOf(CATEGORIES_RULES.bubbles.suspicious, +category.id) !== -1 && category.confidence >= MIN_CATEGORY_CONFIDENCE) {
				attr.safety = SUSPICIOUS;
				attr.color = 'yellow';
			} else if (_.indexOf(CATEGORIES_RULES.bubbles.safe, +category.id) !== -1 && category.confidence >= MIN_CATEGORY_CONFIDENCE && attr.safety !== SUSPICIOUS) {
				attr.safety = SAFE;
				attr.color = 'green';
			}
		}
		return attr;
	}
	//cast old format into new
	parseResponseRating(target, host) {
		let result = { categories: [] };
		if (target.application && target.application[0]) {
			result.rating = target.application[0]["@attributes"].r;
			result.confidence = target.application[0]["@attributes"].c;
		} else {
			result.rating = -1;
			result.confidence = -1;
		}
		if (target.application && target.application[3]) {
			result.childRating = target.application[3]["@attributes"].r;
			result.childConfidence = target.application[3]["@attributes"].c;
		} else {
			result.childRating = -1;
			result.childConfidence = -1;
		}
		result.childSafety = result.childRating < MIN_CHILD_RATING && result.childConfidence > MIN_CHILD_RATING_CONFIDENCE ? NOT_SAFE : SAFE;
		if (target.category) {
			if (target.category[0]) {
				for (let index in target.category) {
					let category = target.category[index]["@attributes"];
					if (category.c >= MIN_CATEGORY_CONFIDENCE) {
						result.categories.push(_.merge({}, ALL_CATEGORIES[category.name], { confidence: category.c }));
					}
				}
			} else {
				let category = target.category["@attributes"];
				if (category.c >= MIN_CATEGORY_CONFIDENCE) {
					result.categories.push(_.merge({}, ALL_CATEGORIES[category.name], { confidence: category.c }));
				}
			}
		}
		return _.merge({}, result, this.parseCategories(result.categories), {
			host: host,
			updated: +new Date()
		});
	}
	//prepare review as well
	parseResponseReview(target) {
		let result = { rating: -1, childSafety: -1, safetyCategories: [], childSafetyCategories: [], reputationCategories: [], step: 0 };
		if (target.application && target.application[0] && target.application[0]["@attributes"].t) {
			result.rating = +target.application[0]["@attributes"].t;
			result.step = 6;
		}
		if (target.category) {
			if (target.category[0]) {
				for (let index in target.category) {
					let category = target.category[index]["@attributes"];
					if (category.vote && category.vote == 1) {
						result.step = 6;
						switch (true) {
							case CATEGORIES_RULES.ratingFlow.safety.indexOf(category.name) !== -1:
								result.safetyCategories.push(+category.name);
								break;
							case CATEGORIES_RULES.ratingFlow.childSafety.indexOf(category.name) !== -1:
								result.childSafetyCategories.push(+category.name);
								break;
							case CATEGORIES_RULES.ratingFlow.reputation.indexOf(category.name) !== -1:
								result.reputationCategories.push(+category.name);
								break;
							default:
								break;
						}
					}
				}
			} else {
				let category = target.category["@attributes"];
				if (category.vote && category.vote == 1) {
					result.step = 6;
					switch (true) {
						case CATEGORIES_RULES.ratingFlow.safety.indexOf(category.name) !== -1:
							result.safetyCategories.push(+category.name);
							break;
						case CATEGORIES_RULES.ratingFlow.childSafety.indexOf(category.name) !== -1:
							result.childSafetyCategories.push(+category.name);
							break;
						case CATEGORIES_RULES.ratingFlow.reputation.indexOf(category.name) !== -1:
							result.reputationCategories.push(+category.name);
							break;
						default:
							break;
					}
				}
			}
		}
		if (target.application && target.application[3] && target.application[3]["@attributes"].t) {
			result.step = 6;
			result.childSafety = _.intersection(result.childSafetyCategories, CATEGORIES_RULES.ratingFlow.childSafety).length === 0;
		}
		return result;
	}
	//thats always false for now
	getAdditionalTargetParams() {
		let r = {
			"epochtime": new Date() / 1,
			"id": this.getRtId()
		};
		if (typeof _wotPloyder !== 'undefined' && _wotPloyder.hasItems()) {
			r.lk = _wotPloyder.takeAndPack(_wotPloyder.limit);
		}
		return r;
	}
	subsfwrdToQueryString(subsfwrdArray, options) {
		if (!subsfwrdArray || subsfwrdArray.length === 0) {
			return "";
		}
		let components = [];
		for (let i in subsfwrdArray) {
			let value = subsfwrdArray[i];
			components.push(this.paramsToQuerystring({ subsfwrd: value }, options));
		}
		return components.join("&");
	}
	targetParamsToQuerystring(targetParams, options) {
		let subsfwrdQuerystring = this.subsfwrdToQueryString(targetParams.subsfwrd, options);
		if (targetParams.hasOwnProperty("subsfwrd")) {
			delete targetParams.subsfwrd;
		}
		let queryString = this.paramsToQuerystring(targetParams, options);
		if (subsfwrdQuerystring) {
			queryString += "&" + subsfwrdQuerystring;
		}
		return queryString;
	}
	paramsToQuerystring(params, options) {
		if (!params) {
			return "";
		}
		let components = [];
		for (let i in params) {
			if (params[i] != null) {
				let paramName = i,
				    paramValue = params[i];
				// Use a hash instead of the real value in the authenticated query
				if (options && options.hash && options.hash == i) {
					paramName = "SHA1";
					paramValue = this.crypto.bintohex(this.crypto.sha1.sha1str(unescape(encodeURIComponent(params[i]))));
				}
				components.push(paramName + "=" + encodeURIComponent(paramValue));
			}
		}
		return components.join("&");
	}
	encodehostname(host, path) {
		try {
			if (!host || !path) {
				return host;
			}
			/* Clean up the path, drop query string and hash */
			path = path.replace(/^\s*/g, "").replace(/\s*$/, "").replace(/[\?#].*$/, "").replace(/\/$/, "");
			if (path.length < 2 || path[0] != "/") {
				return host;
			}
			let c = path.split("/");
			if (!c || !c.length) {
				return host;
			}
			/* Drop a suspected filename from the end */
			if (path[path.length - 1] != "/" && /\.[^\.]{1,6}$/.test(c[c.length - 1])) {
				c.pop();
			}
			let level = 0;
			for (let i = c.length; !level && i > 0; --i) {
				level = this.getsharedlevel(host + c.slice(0, i).join("/"));
			}
			if (!level) {
				return host;
			}
			let p = c.slice(0, level + 1).join("/").replace(/^\//, "");
			if (!p || !p.length) {
				return host;
			}
			let encoded = this.crypto.base32.encode(p);
			if (encoded == null) {
				return host;
			}
			return "_p_" + encoded + "." + host;
		} catch (e) {
			;
		}
		return host;
	}
	getsharedlevel(host) {
		return SHARED_HOSTS[host] || 0;
	}
	getRtId() {
		// get the "stats_uid" if such exists for opera 
		if (browserName === PLATFORM_TYPES.OPERA) {
			let statsUid = JSON.parse(localStorage.getItem("stats_uid"));
			if (statsUid) {
				return statsUid;
			}
		}
		let uuid = JSON.parse(localStorage.getItem("uuid"));
		if (uuid) {
			return uuid;
		}
		if (this.witness && this.witness.id) {
			return this.witness.id;
		}
		throw new Error("WotApi.getRtId: RT id is not set");
	}
}
let WotApi = new _WotApi();