//bunch of utilities
//just to make sure, that everything is secure
function strtobin(str) {
	var bin = [];
	for (var i = 0; i < str.length; ++i) {
		bin[i] = str.charCodeAt(i) & 0xFF;
	}
	return bin;
}
function bintostr(bin) {
	var str = "";
	for (var i = 0; i < bin.length; ++i) {
		str += String.fromCharCode(bin[i] & 0xFF);
	}
	return str;
}
function hextobin(str) {
	var asciitonibble = function (c) {
		var code_a = 'a'.charCodeAt(0);
		if (c >= code_a) {
			return c - code_a + 10;
		} else {
			return c - '0'.charCodeAt(0);
		}
	};
	var bin = [];
	for (var i = 0; i < str.length / 2; ++i) {
		bin[i] = asciitonibble(str.charCodeAt(2 * i)) << 4 | asciitonibble(str.charCodeAt(2 * i + 1)) & 0xF;
	}
	return bin;
}
function bintohex(bin) {
	const HEX = "0123456789abcdef";
	var str = "";
	for (var i = 0; i < bin.length; ++i) {
		str += HEX.charAt(bin[i] >> 4 & 0xF);
		str += HEX.charAt(bin[i] & 0xF);
	}
	return str;
}
const sha1 = {
	hmacsha1hex: function (hexkey, str) {
		var key = hextobin(hexkey);
		if (key.length > 20) {
			key = this.sha1bin(key);
		}
		var ipad = Array(64),
		    opad = Array(64);
		for (var i = 0; i < 20; ++i) {
			ipad[i] = key[i] ^ 0x36;
			opad[i] = key[i] ^ 0x5C;
		}
		for (var j = 20; j < 64; ++j) {
			ipad[j] = 0x36;
			opad[j] = 0x5C;
		}
		var inner = this.sha1bin(ipad.concat(strtobin(str)));
		return this.sha1bin(opad.concat(inner));
	},
	sha1bin: function (bin) {
		return this.sha1str(bintostr(bin));
	},
	sha1hex: function (hex) {
		return this.sha1str(hextostr(hex));
	},
	/* A JavaScript implementation of the Secure Hash Algorithm, SHA-1,
  as defined in FIPS PUB 180-1
  Version 2.1a Copyright Paul Johnston 2000 - 2002.
  Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
  Distributed under the BSD License
  See http://pajhome.org.uk/crypt/md5 for details. */
	sha1str: function (str) {
		var ft = function (t, b, c, d) {
			if (t < 20) {
				return b & c | ~b & d;
			}
			if (t >= 40 && t < 60) {
				return b & c | b & d | c & d;
			}
			return b ^ c ^ d;
		};
		var kt = function (t) {
			if (t < 20) {
				return 1518500249;
			}
			if (t < 40) {
				return 1859775393;
			}
			if (t < 60) {
				return -1894007588;
			}
			return -899497514;
		};
		var add = function (x, y) {
			var lsw = (x & 0xFFFF) + (y & 0xFFFF);
			var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
			return msw << 16 | lsw & 0xFFFF;
		};
		var rol = function (num, cnt) {
			return num << cnt | num >>> 32 - cnt;
		};
		var len = str.length * 8;
		var bin = [];
		var mask = (1 << 8) - 1;
		for (var i = 0; i < len; i += 8) {
			bin[i >> 5] |= (str.charCodeAt(i / 8) & mask) << 24 - i % 32;
		}
		bin[len >> 5] |= 0x80 << 24 - len % 32;
		bin[(len + 64 >> 9 << 4) + 15] = len;
		var w = Array(80);
		var a = 1732584193;
		var b = -271733879;
		var c = -1732584194;
		var d = 271733878;
		var e = -1009589776;
		for (var i = 0; i < bin.length; i += 16) {
			var olda = a;
			var oldb = b;
			var oldc = c;
			var oldd = d;
			var olde = e;
			for (var j = 0; j < 80; j++) {
				if (j < 16) {
					w[j] = bin[i + j];
				} else {
					w[j] = rol(w[j - 3] ^ w[j - 8] ^ w[j - 14] ^ w[j - 16], 1);
				}
				var t = add(add(rol(a, 5), ft(j, b, c, d)), add(add(e, w[j]), kt(j)));
				e = d;
				d = c;
				c = rol(b, 30);
				b = a;
				a = t;
			}
			a = add(a, olda);
			b = add(b, oldb);
			c = add(c, oldc);
			d = add(d, oldd);
			e = add(e, olde);
		}
		bin = [a, b, c, d, e];
		str = "";
		for (var i = 0; i < bin.length * 32; i += 8) {
			str += String.fromCharCode(bin[i >> 5] >>> 24 - i % 32 & mask);
		}
		return strtobin(str);
	}
};
const arc4 = {
	create: function (key) {
		var i,
		    j = 0,
		    k = 0,
		    l;
		var ctx = {};
		ctx.s = [];
		ctx.x = 1;
		ctx.y = 0;
		for (i = 0; i < 256; ++i) {
			ctx.s[i] = i;
		}
		for (i = 0; i < 256; ++i) {
			l = ctx.s[i];
			j = j + key[k] + l & 0xFF;
			ctx.s[i] = ctx.s[j];
			ctx.s[j] = l;
			if (++k >= key.length) {
				k = 0;
			}
		}
		return ctx;
	},
	crypt: function (ctx, input) {
		var i, j, k;
		var output = [];
		for (i = 0; i < input.length; ++i) {
			j = ctx.s[ctx.x];
			ctx.y = ctx.y + j & 0xFF;
			k = ctx.s[ctx.y];
			ctx.s[ctx.x] = k;
			ctx.s[ctx.y] = j;
			ctx.x = ctx.x + 1 & 0xFF;
			output[i] = (input[i] ^ ctx.s[j + k & 0xFF]) & 0xFF;
		}
		return output;
	}
};
const base32 = {
	set: "abcdefghijklmnopqrstuvwxyz234567",
	encode: function (s) {
		try {
			/* Unicode to UTF-8 */
			s = unescape(encodeURIComponent(decodeURIComponent(s)));
			var r = "";
			var b = 0;
			var l = 0;
			for (var i = 0; i < s.length; ++i) {
				var n = s.charCodeAt(i);
				if (n > 255) {
					return null;
					/* Invalid input */
				}
				b = (b << 8) + n;
				l += 8;
				do {
					l -= 5;
					r += this.set[b >> l & 0x1F];
				} while (l >= 5);
			}
			if (l > 0) {
				r += this.set[b << 5 - l & 0x1F];
			}
			return r;
		} catch (e) {
			console.log("base32.encode: failed with " + e + "\n");
		}
		return null;
	},
	decode: function (s) {
		try {
			/* Build a reverse lookup table */
			if (!this.rev) {
				this.rev = {};
				for (var i = 0; i < this.set.length; ++i) {
					this.rev[this.set.charAt(i)] = i;
				}
			}
			var r = "";
			var b = 0;
			var l = 0;
			for (var i = 0; i < s.length; ++i) {
				var n = this.rev[s.charAt(i)];
				if (n == null) {
					return null;
					/* Invalid input */
				}
				b = (b << 5) + n;
				l += 5;
				while (l >= 8) {
					l -= 8;
					r += String.fromCharCode(b >> l & 0xFF);
				}
			}
			if (l >= 5) {
				return null;
				/* Invalid input */
			}
			/* UTF-8 to Unicode */
			return decodeURIComponent(escape(r));
		} catch (e) {
			console.log("base32.decode: failed with " + e + "\n");
		}
		return null;
	}
};
class WotCrypto {
	constructor(counter) {
		this.counter = counter || 0;
		this.sha1 = sha1;
		this.arc4 = arc4;
		this.bintohex = bintohex;
		this.base32 = base32;
	}
	getnonce(seed, witness) {
		/* must be unique, not necessarily unpredictable */
		let data = "nonce:" + getVersion() + ":" + window.navigator.language + ":" + this.counter++ + ":" + Date.now() + ":" + Math.random() + ":" + ((witness || { id: "", key: "" }).id || "") + ":" + (seed || "");
		return bintohex(this.sha1.sha1str(data));
	}
	decrypt(data, nonce, options) {
		try {
			if (data && nonce) {
				var key = options && options.witness_key ? options.witness_key : this.witness.key;
				var index = (options || {}).index;
				var response_str = (options || {}).response_str;
				if (index == undefined || index < 0) {
					index = "";
				} else {
					index = "-" + index;
				}
				var hash_string = nonce + index;
				if (response_str) {
					hash_string = "response-" + hash_string;
				}
				//
				// debug -
				/*var atob_result = atob(data);
    var strtobin = this.strtobin(atob_result);
    var sha1 = this.sha1.hmacsha1hex(key, nonce);
    var arc4create = this.arc4.create(sha1);
    var arc4crypt = this.arc4.crypt(arc4create, strtobin);
    var result = this.bintostr(arc4crypt);
    console.log(result);*/
				if (key) {
					return bintostr(this.arc4.crypt(this.arc4.create(this.sha1.hmacsha1hex(key, hash_string)), strtobin(atob(data))));
				}
			}
		} catch (e) {
			console.log("crypto.decrypt: failed with " + e + "\n");
		}
		return null;
	}
	encrypt(data, nonce, key, witness) {
		try {
			if (data && nonce) {
				if (!key) {
					var key = (witness || {}).key;
				}
				if (key) {
					// debug -
					// var sha1 = this.sha1.hmacsha1hex(key, nonce);
					// var strtobin = this.strtobin(data);
					// var arc4create = this.arc4.create(sha1);
					// var arc4crypt = this.arc4.crypt(arc4create, strtobin);
					// var bintostr = this.bintostr(arc4crypt);
					// var result = btoa(bintostr);
					return btoa(bintostr(this.arc4.crypt(this.arc4.create(this.sha1.hmacsha1hex(key, nonce)), strtobin(data))));
				}
			}
		} catch (e) {
			console.log("crypto.encrypt: failed with " + e + "\n");
		}
		return null;
	}
	authenticate(data, witness) {
		try {
			var key = (witness || this.witness || {}).key;
			if (key) {
				return bintohex(this.sha1.hmacsha1hex(key, data));
			}
		} catch (e) {
			console.log("crypto.authenticate: failed with " + e + "\n");
		}
		return null;
	}
	sha1str(str) {
		var ft = function (t, b, c, d) {
			if (t < 20) {
				return b & c | ~b & d;
			}
			if (t >= 40 && t < 60) {
				return b & c | b & d | c & d;
			}
			return b ^ c ^ d;
		};
		var kt = function (t) {
			if (t < 20) {
				return 1518500249;
			}
			if (t < 40) {
				return 1859775393;
			}
			if (t < 60) {
				return -1894007588;
			}
			return -899497514;
		};
		var add = function (x, y) {
			var lsw = (x & 0xFFFF) + (y & 0xFFFF);
			var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
			return msw << 16 | lsw & 0xFFFF;
		};
		var rol = function (num, cnt) {
			return num << cnt | num >>> 32 - cnt;
		};
		var len = str.length * 8;
		var bin = [];
		var mask = (1 << 8) - 1;
		for (var i = 0; i < len; i += 8) {
			bin[i >> 5] |= (str.charCodeAt(i / 8) & mask) << 24 - i % 32;
		}
		bin[len >> 5] |= 0x80 << 24 - len % 32;
		bin[(len + 64 >> 9 << 4) + 15] = len;
		var w = Array(80);
		var a = 1732584193;
		var b = -271733879;
		var c = -1732584194;
		var d = 271733878;
		var e = -1009589776;
		for (var i = 0; i < bin.length; i += 16) {
			var olda = a;
			var oldb = b;
			var oldc = c;
			var oldd = d;
			var olde = e;
			for (var j = 0; j < 80; j++) {
				if (j < 16) {
					w[j] = bin[i + j];
				} else {
					w[j] = rol(w[j - 3] ^ w[j - 8] ^ w[j - 14] ^ w[j - 16], 1);
				}
				var t = add(add(rol(a, 5), ft(j, b, c, d)), add(add(e, w[j]), kt(j)));
				e = d;
				d = c;
				c = rol(b, 30);
				b = a;
				a = t;
			}
			a = add(a, olda);
			b = add(b, oldb);
			c = add(c, oldc);
			d = add(d, oldd);
			e = add(e, olde);
		}
		bin = [a, b, c, d, e];
		str = "";
		for (var i = 0; i < bin.length * 32; i += 8) {
			str += String.fromCharCode(bin[i >> 5] >>> 24 - i % 32 & mask);
		}
		return strtobin(str);
	}
}